import pytest
from pytest_alembic import MigrationContext

from sqlalchemy_declarative_extensions.dialects import get_roles
from sqlalchemy_declarative_extensions.dialects.postgresql import Role


def test_check_revisions(alembic_runner: MigrationContext, alembic_engine):
    with pytest.raises(RuntimeError, match=r"^New upgrade operations detected"):
        alembic_runner.raw_command("check")


def test_apply_autogenerated_revision(alembic_runner: MigrationContext, alembic_engine):
    result = alembic_runner.generate_revision(
        autogenerate=True, prevent_file_generation=False
    )
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        result = get_roles(conn, exclude=[alembic_engine.pmr_credentials.username])

    expected_result = [
        Role(
            "admin",
            superuser=True,
            createdb=True,
            createrole=True,
            login=False,
            replication=True,
            bypass_rls=True,
            in_roles=["read", "write"],
        ),
        Role(
            "app",
            login=True,
            in_roles=["read", "write"],
        ),
        Role("read"),
        Role("write"),
    ]
    assert expected_result == result
