import pytest
from pytest_alembic import MigrationContext
from sqlalchemy import text
from sqlalchemy.exc import ProgrammingError


def test_apply_autogenerated_revision(alembic_runner: MigrationContext, alembic_engine):
    alembic_runner.migrate_up_to("head")

    # Ensure we actually inserted the rows at the correct revision.
    with alembic_engine.connect() as conn:
        result = conn.execute(text("select * from foo")).fetchall()

    expected_result = [
        (2, "asdf", None),
        (3, "qwer", False),
    ]
    assert expected_result == result

    # Drops the table
    alembic_runner.generate_revision(autogenerate=True, prevent_file_generation=False)
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        with pytest.raises(ProgrammingError):
            conn.execute(text("select * from foo")).fetchall()
