from sqlalchemy import text

from sqlalchemy_declarative_extensions.dialects import get_roles
from sqlalchemy_declarative_extensions.dialects.postgresql import Role


def test_apply_autogenerated_revision(alembic_runner, alembic_engine):
    with alembic_engine.connect() as conn:
        conn.execute(text("CREATE ROLE meow"))
        conn.execute(text("CREATE ROLE admin IN ROLE meow"))
        conn.execute(text("CREATE ROLE extra_role"))

    result = alembic_runner.generate_revision(
        autogenerate=True, prevent_file_generation=False
    )
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        result = get_roles(conn, exclude=[alembic_engine.pmr_credentials.username])

    expected_result = [
        Role(
            "admin",
            superuser=False,
            createdb=True,
            createrole=True,
            login=True,
            replication=True,
            bypass_rls=True,
            valid_until=None,
            in_roles=["read"],
        ),
        Role(
            "read",
            superuser=False,
            createdb=False,
            createrole=False,
            login=False,
            replication=False,
            bypass_rls=False,
            valid_until=None,
        ),
    ]
    assert result == expected_result
