from pytest_alembic import MigrationContext
from sqlalchemy import text


def test_apply_autogenerated_revision(alembic_runner: MigrationContext, alembic_engine):
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        conn.execute(
            text("INSERT INTO foo (id, name, active) VALUES (2, 'changeme', True)")
        )
        conn.execute(
            text("INSERT INTO foo (id, name, active) VALUES (3, 'changeme', True)")
        )
        conn.execute(text("commit"))

    alembic_runner.generate_revision(autogenerate=True, prevent_file_generation=False)
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        result = conn.execute(text("select * from foo")).fetchall()

    expected_result = [
        # The `True` value here is left unspecified in the declarative definition
        (2, "asdf", True),
        # Whereas this one **is** specified on the definition and overridden
        (3, "qwer", False),
    ]
    assert expected_result == result
