import pytest
from pytest_alembic import MigrationContext
from pytest_mock_resources import MysqlConfig, create_mysql_fixture
from sqlalchemy import text
from sqlalchemy.exc import ProgrammingError

alembic_engine = create_mysql_fixture(scope="function")


@pytest.fixture(scope="session")
def pmr_mysql_config():
    return MysqlConfig(image="mysql:8", port=None, ci_port=None)


def test_apply_autogenerated_revision(alembic_runner: MigrationContext, alembic_engine):
    alembic_runner.generate_revision(autogenerate=True, prevent_file_generation=False)
    alembic_runner.migrate_up_one()

    with alembic_engine.connect() as conn:
        with conn.begin() as trans:
            conn.execute(text("CREATE VIEW bar AS select * from foo"))
            trans.commit()

    alembic_runner.generate_revision(autogenerate=True, prevent_file_generation=False)
    alembic_runner.migrate_up_one()

    with pytest.raises(ProgrammingError):
        with alembic_engine.connect() as conn:
            conn.execute(text("select * from bar"))
